@@ -35,9 +35,11 @@ class Match:
         self.__actions_hashes = set()
         if "name" in args:
             self._name = args["name"]
-        if "icon" in args:
-            self._icon = args["icon"]
-        if "pixbuf" in args and isinstance(args["pixbuf"], gtk.gdk.Pixbuf):
+        if "icon" in args and args["icon"] != None:
+            self.set_icon(args["icon"])
+        if "pixbuf" in args and args["pixbuf"] != None:
+            if not isinstance(args["pixbuf"], gtk.gdk.Pixbuf):
+                raise TypeError, "pixbuf must be a gtk.gdk.Pixbuf"
             self._pixbuf = args["pixbuf"]
         if "category" in args:
             self._category = args["category"]
@@ -90,6 +92,8 @@ class Match:
         
         @type iconname: string 
         """
+        if not isinstance(iconname, str):
+            raise TypeError, "icon must be a string"
         self._icon = iconname
     
     def get_category(self):
@@ -154,10 +158,10 @@ class Match:
         Matches that have same hash will be selected based on the handler priority.
         text is the entered query string.
         
-        By default, if the handler does not override this, it will return None.
-        Returning None means no duplication check will be performed.
+        By default, if the handler does not override this, it will return the
+        id of the class.
         """
-        return None
+        return id(self)
     
     def get_name(self, text=None):
         """
