@@ -24,6 +24,7 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         self.__small_window_height = None
         self._do_clear = True
         
+        self.connect("configure-event", self.__save_window_size)
         self.connect("delete-event", self._controller.on_quit)
         self.connect("destroy-event", self._controller.on_quit)
         self.connect("key-press-event", self.__on_window_key_press_event)
@@ -32,7 +33,6 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         self.set_default_size( self._model.get_window_width(), -1 )
         self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
         self.set_property("skip-taskbar-hint", True)
-        self.set_position(gtk.WIN_POS_CENTER_ALWAYS)
 
         self._model.connect("query-ready", lambda s,m: gobject.idle_add(self.append_matches, s, m))
         
@@ -139,6 +139,7 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
        
         # Results
         self.results_box = gtk.HBox()
+        self.results_box.connect("unmap", self.__save_window_height)
         self.results_box.pack_start(self.scrolled_results)
         self.results_box.pack_start(self.actions_box)
         self.vbox_main.pack_start(self.results_box)
@@ -146,8 +147,6 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
     def clear_all(self):
         deskbar.interfaces.View.clear_all(self)
         width, height = self.get_size()
-        self._model.set_window_width( width )
-        self._model.set_window_height( height )
         
         if self.__small_window_height != None:
             self.resize( width, self.__small_window_height )
@@ -181,6 +180,7 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         self.realize()
         self.window.set_user_time(time)
         self.present()
+        self.move( self._model.get_window_x(), self._model.get_window_y() )
     
     def __show_matches(self):
         self.scrolled_results.show()
@@ -192,8 +192,6 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
     
     def show_results(self):
         width, height = self.get_size()
-        if self.__small_window_height == None:
-            self.__small_window_height = height
         self.results_box.show()
         self.__show_matches()
         self.resize( width, self._model.get_window_height() )
@@ -249,8 +247,6 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
     
     def __on_window_key_press_event(self, window, event):
         if event.keyval == gtk.keysyms.Escape:
-            if self._model.get_clear_entry():
-                self.clear_all()
             self.emit("destroy-event", event)
                 
         return False
@@ -262,6 +258,21 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
             elif mode == "bottom":
                 self.cview.select_last_item()
             self.cview.grab_focus()
+            return True
         else:
-            self.entry.grab_focus()
-        
\ No newline at end of file
+            return False
+        
+    def __save_window_size(self, window, event):
+        """
+        Save window width and height of the window when
+        results_box is not visible
+        """
+        self._model.set_window_width( event.width )
+        if self.__small_window_height == None:
+            self.__small_window_height = event.height
+            
+    def __save_window_height(self, resultsbox):
+        """
+        Save window height before results_box disappears
+        """
+        self._model.set_window_height( self.get_size()[1] )
\ No newline at end of file
