@@ -29,10 +29,13 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
                  'has-more-actions' : (gobject.TYPE_BOOLEAN, 'whether the match has more than one action',
                                     'If set to True a symbol will be displayed on the right',
                                     False, gobject.PARAM_READWRITE),
-                 'match-markup' : (gobject.TYPE_STRING, 'markup for match description',
-                      'markup for match description',
-                      "", gobject.PARAM_READWRITE),
         }
+    __gsignals__ = {
+        # This signal will be emited when '>' on the right is clicked
+        "show-actions-activated": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT]),
+        # This signal will be emited then an area that's not the '>' from above is clicked
+        "do-action-activated": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT]),
+    }
     
     def __init__ (self):
         gtk.CellRendererText.__init__ (self)
@@ -40,14 +43,21 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
         self.__match_count = 0
         self.__has_more_actions = False
         self.__match_markup = ""
+        self.__button_x = 0
+        self.__button_y = 0
+        self.__button_width = 0
+        self.__button_height = 0
+        self.__relative_header_size = -0.2 # Make header 20% smaller than normal fonts
+        
+        self.set_property("mode", gtk.CELL_RENDERER_MODE_ACTIVATABLE)
         
         # Grab some default theme settings
         # they are probably incorrect, but we reset
         # them on each render anyway.
         style = gtk.Style ()
         self.header_font_desc = style.font_desc
-        self.header_font_desc.set_weight (pango.WEIGHT_BOLD)
-        self.header_font_desc.set_size (self.header_font_desc.get_size () - pango.SCALE *2)
+        self.header_font_desc.set_weight (pango.WEIGHT_BOLD)        
+        self.header_font_desc.set_size (self.header_font_desc.get_size () + int(self.header_font_desc.get_size ()*self.__relative_header_size))
         self.header_bg = style.base [gtk.STATE_NORMAL]
     
     def set_style (self, widget):
@@ -56,7 +66,7 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
         """
         self.header_font_desc = widget.style.font_desc
         self.header_font_desc.set_weight (pango.WEIGHT_BOLD)
-        self.header_font_desc.set_size (self.header_font_desc.get_size () - pango.SCALE *2)
+        self.header_font_desc.set_size (self.header_font_desc.get_size () + int(self.header_font_desc.get_size ()*self.__relative_header_size))
         self.header_bg = widget.style.base [gtk.STATE_NORMAL]
     
     def do_render (self, window, widget, background_area, cell_area, expose_area, flags):
@@ -64,33 +74,54 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
             self.render_match (window, widget, background_area, cell_area, expose_area, flags)
         else:
             self.render_category (window, widget, background_area, cell_area, expose_area, flags)
-    
-    def render_match (self, window, widget, background_area, cell_area, expose_area, flag):
+   
+    def render_match (self, window, widget, background_area, cell_area, expose_area, flags):
         ctx = window.cairo_create ()
         
         # Set up a pango.Layout
         more_actions_layout = ctx.create_layout ()
         if self.get_property("has-more-actions"):
             more_actions_layout.set_markup ("<b>&gt;</b>")
-        else:
-            more_actions_layout.set_text("")
+        
         more_actions_layout.set_font_description (self.header_font_desc)
         
         more_actions_layout_width, more_actions_layout_height = more_actions_layout.get_pixel_size()
                 
-        state = self.renderer_state_to_widget_state(flag)
+        state = self.renderer_state_to_widget_state(flags)
+        
+        self.__button_width = more_actions_layout_width + 2
+        self.__button_height = cell_area.height 
         
         # Draw the actual text in the remaining area
         cell_area_width = cell_area.width
-        cell_area.width -= more_actions_layout_width + 2
-        gtk.CellRendererText.do_render (self, window, widget, background_area, cell_area, expose_area, flag)
+        cell_area.width -= self.__button_width
+        gtk.CellRendererText.do_render (self, window, widget, background_area, cell_area, expose_area, flags)
         
         mod_gc = widget.get_style().text_gc[state]
+        
+        self.__button_x = (cell_area.x + cell_area_width) - more_actions_layout_width - 2
+        self.__button_y = cell_area.y + ( (cell_area.height - more_actions_layout_height) / 2 ) + 1
+        
         window.draw_layout(mod_gc,
-                        (cell_area.x + cell_area_width) - more_actions_layout_width - 2,
-                        cell_area.y + ( (cell_area.height - more_actions_layout_height) / 2 ) + 1,
+                        self.__button_x,
+                        self.__button_y,
                         more_actions_layout)
         
+        if self.get_property("has-more-actions"):
+            # Add some additional area around the '>' to activate it
+            self.__button_x -= 3
+            self.__button_y = cell_area.y
+            self.__button_width += 3
+        else:
+            self.__button_height = 0
+            self.__button_width = 0
+            self.__button_x = 0
+            self.__button_y = 0
+        
+        #window.draw_rectangle(mod_gc, False, self.__button_x, self.__button_y,
+        #            self.__button_width, self.__button_height          
+        #            )
+        
     def render_category (self, window, widget, background_area, cell_area, expose_area, flag):
         """
         Renders the category title from the "category-header" property and displays a rigth aligned
@@ -105,7 +136,7 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
         cat_layout = ctx.create_layout ()
         cat_layout.set_font_description (self.header_font_desc)
         
-        cat_layout.set_markup("...")        
+        cat_layout.set_markup("...")
         cat_layout_width, cat_layout_height = cat_layout.get_pixel_size()
         ellipsise_size = cat_layout_width
         
@@ -153,6 +184,33 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
         if (gtk.CELL_RENDERER_SELECTED & flags) == gtk.CELL_RENDERER_SELECTED:
             state = gtk.STATE_SELECTED
         return state
+    
+    def do_activate(self, event, widget, path_string, background_area, cell_area, flags):
+        if not isinstance(widget, gtk.TreeView):
+            # Not a treeview
+            return False
+        
+        if event == None or event.type != gtk.gdk.BUTTON_PRESS:
+            # Event type not GDK_BUTTON_PRESS
+            return True
+        
+        ex = event.x
+        ey = event.y
+        bx = self.__button_x
+        # Otherwise, we get problems when the cell
+        # is at the top of the visible part of the treeview
+        by = cell_area.y 
+        bw = self.__button_width
+        bh = self.__button_height
+        
+        path = tuple([int(i) for i in path_string.split(':')])
+        if (ex < bx or ex > (bx+bw) or ey < by or ey > (by+bh)):
+            # Click wasn't on the icon
+            self.emit("do-action-activated", path)
+            return True
+        else:
+            self.emit("show-actions-activated", path)
+            return False
         
     def do_get_property(self, property):
         if property.name == 'category-header':
@@ -161,8 +219,6 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
             return self.__match_count
         elif property.name == 'has-more-actions':
             return self.__has_more_actions
-        elif property.name == 'match-markup':
-            return self.__match_markup
         else:
             raise AttributeError, 'unknown property %s' % property.name
 
@@ -173,8 +229,6 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
             self.__match_count = value
         elif property.name == 'has-more-actions':
             self.__has_more_actions = value
-        elif property.name == 'match-markup':
-            self.__match_markup = value
         else:
             raise AttributeError, 'unknown property %s' % property.name
         
@@ -190,6 +244,8 @@ class CuemiacTreeView (gtk.TreeView):
     __gsignals__ = {
         "match-selected" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_STRING, gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT]),
         "do-default-action" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_STRING, gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT]),
+        "pressed-up-at-top" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
+        "pressed-down-at-bottom" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
     }
     
     def __init__ (self, model):
@@ -197,15 +253,15 @@ class CuemiacTreeView (gtk.TreeView):
         self.set_enable_search (False)
         self.set_property ("headers-visible", False)
         
-        self.connect ("row-activated", self.__on_activated) # Used activate result if enter in entry has been pressed 
-        self.connect ("button-press-event", self.__on_button_press)        
         self.connect ("key-press-event", self.__on_key_press)
                 
         icon = gtk.CellRendererPixbuf ()
         icon.set_property("xpad", 10)
         hit_title = CellRendererCuemiacCategory ()
+        hit_title.connect("show-actions-activated", self.__on_show_actions_activated)
+        hit_title.connect("do-action-activated", self.__on_do_action_activated)
         hit_title.set_property ("ellipsize", pango.ELLIPSIZE_END)
-        
+       
         hits = gtk.TreeViewColumn ("Hits")
         hits.pack_start (icon, expand=False)
         hits.pack_start (hit_title)
@@ -217,11 +273,7 @@ class CuemiacTreeView (gtk.TreeView):
         # FIXME: Make it so that categories *only* can be reordered by dragging
         # gtkTreeView does not use normal gtk DnD api.
         # it uses the api from hell
-    
-    def grab_focus_and_select_first(self):
-        self.__select_first_item()
-        gtk.TreeView.grab_focus(self)
-    
+
     def is_ready (self):
         """ Returns True if the view is ready for user interaction """
         num_children = self.get_model().iter_n_children (None)
@@ -265,19 +317,37 @@ class CuemiacTreeView (gtk.TreeView):
         else:
             return False
 
-    def __on_button_press (self, treeview, event):
-        # We want to activate items on single click
-        path_ctx = self.get_path_at_pos (int(event.x), int(event.y))
-        if path_ctx is not None:
-            path, col, x, y = path_ctx
-            model = self.get_model ()
-            match = model[model.get_iter(path)][model.MATCHES]
-            if match.__class__ != CuemiacCategory:
-                if event.state & gtk.gdk.CONTROL_MASK:
-                    self.__on_activated(treeview, path, col, event)
-                else:
-                    self.__on_do_default_action(treeview, path, col, event)
-    
+    def activate_first_item(self, scroll=True):
+        """
+        Activate to the first item and scroll to it if C{scroll} is C{True}
+        """
+        path = self.__find_top_path()
+        if path != None:
+            if scroll:
+                self.__select_path(path)
+            self.activate_row( self.get_model().get_iter(path) )
+
+    def activate_row(self, iter):
+        """
+        Doesn't scroll to the cell. We pretend
+        that the row is already selected.
+        """
+        path = self.get_model().get_path(iter)
+        if path != None:
+            col = self.get_column(0)
+            self.__on_activated(self, path, col)
+
+    def select_first_item(self):
+        model = self.get_model()
+        path = self.__find_top_path()
+        if path != None:
+            self.__select_path(path)
+        
+    def select_last_item(self):
+        path = self.__find_bottom_path()
+        if path != None:
+            self.__select_path(path)
+
 #    def __on_config_expanded_cat (self, value):
 #        if value != None and value.type == gconf.VALUE_LIST:
 #            self.__collapsed_rows = [h.get_string() for h in value.get_list()]
@@ -296,7 +366,7 @@ class CuemiacTreeView (gtk.TreeView):
                 cell.set_property ("pixbuf", match.get_icon())
                 cell.set_property ("visible", True)
             else:
-                logging.error("See bug 359251 and report this output: Match object of unexpected type: %r - %r" % (match.__class__, match))
+                logging.error("See bug 359251 or 471672 and report this output: Match object of unexpected type: %r - %r" % (match.__class__, match))
                 cell.set_property ("pixbuf", None)
                 cell.set_property ("visible", False)
         
@@ -316,10 +386,25 @@ class CuemiacTreeView (gtk.TreeView):
         cell.set_property ("height", -1)
         cell.set_property ("cell-background-gdk", self.style.base[gtk.STATE_NORMAL])
         
-        cell.set_property ("has-more-actions", len(match.get_actions()) > 1)
-                
-        cell.set_property ("markup", model[iter][model.ACTIONS])
+        if match == None:
+            logging.error("See bug 359251 or 471672 and report this output: Match object of unexpected type: %r - %r" % (match.__class__, match))
+            cell.set_property ("has-more-actions", False)
+            cell.set_property ("markup", "")
+        else:
+            cell.set_property ("has-more-actions", len(match.get_actions()) > 1)
+            cell.set_property ("markup", model[iter][model.ACTIONS])
+        
+    def __on_show_actions_activated(self, widget, path):
+        col = self.get_model().ACTIONS
+        self.__on_activated(self, path, col, None)
 
+    def __on_do_action_activated(self, widget, path):
+        model = self.get_model ()
+        match = model[model.get_iter(path)][model.MATCHES]
+        col = self.get_model().ACTIONS
+        if match.__class__ != CuemiacCategory:
+            self.__on_do_default_action(self, path, col, None)
+                    
     def __on_do_default_action(self, treeview, path, col, event):
         model = self.get_model()
         iter = model.get_iter (path)
@@ -371,47 +456,72 @@ class CuemiacTreeView (gtk.TreeView):
         elif (event.keyval == gtk.keysyms.Down):
             if match.__class__ == CuemiacCategory:
                 return False
-            elif model.get_path(iter) == self.__get_bottom_path():
+            elif model.get_path(iter) == self.__find_bottom_path():
                 # We're at the bottom of the list
-                self.__select_first_item()
+                self.emit("pressed-up-at-top")
                 return True
             else:
                 iter_next = model.iter_next(iter)
                 if iter_next == None:
                     # We're at the last item of a category
                     # Select the first item of the next category
-                    parent = model.iter_parent(iter)
-                    num = model.get_path(parent)[0]+1
-                    self.__select_path( (num, 0) )
+                    next_cat = self.__find_next_cat(iter )
+                    if next_cat == None:
+                        return False
+                    else:
+                        num = model.get_path(next_cat)[0]
+                        self.__select_path( (num, 0) )
                     return True
         elif (event.keyval == gtk.keysyms.Up):
             if match.__class__ == CuemiacCategory:
                 return False
-            elif model.get_path(iter) == (0,0):
+            elif model.get_path(iter) == self.__find_top_path():
                 # We're at the top of the list 
-                self.__select_last_item()
+                self.emit("pressed-down-at-bottom")
                 return True
             elif model.get_path(iter)[1] == 0:
                 # We're at the first item of a category
                 # Select the last item of the previous category
-                parent = model.iter_parent(iter)
-                num = model.get_path(parent)[0]-1
-                prev_cat = model.get_iter( (num,) )
-                child = model.iter_n_children(prev_cat)-1
-                
-                self.__select_path( (num, child) )
-                return True
+                prev_cat = self.__find_previous_cat( iter )
+                if prev_cat == None:
+                    return False
+                else:
+                    num = model.get_path(prev_cat)[0]
+                    child = model.iter_n_children(prev_cat)-1
+                    self.__select_path( (num, child) )
+                    return True
             
         return False
     
-    def __select_first_item(self):
+    def __find_next_cat(self, iter):
+        """
+        Find next expanded category
+        """
         model = self.get_model()
-        path = model.get_path( model.iter_nth_child(model.get_iter_first(), 0))
-        self.__select_path(path)
-        
-    def __select_last_item(self):
-        path = self.__get_bottom_path()
-        self.__select_path(path)
+        next_path = (model.get_path(iter)[0]+1, )
+        if next_path[0] >= len(model):
+            # All categories are collapsed
+            return None
+        next_iter = model.get_iter( next_path )
+        if self.row_expanded(next_path):
+            return next_iter
+        else:
+            return self.__find_next_cat( next_iter )
+    
+    def __find_previous_cat(self, iter):
+        """
+        Find next previous expanded category
+        """
+        model = self.get_model()
+        prev_path = (model.get_path(iter)[0]-1, )
+        if prev_path[0] < 0:
+            # All categories are collapsed
+            return None
+        prev_iter = model.get_iter(prev_path)
+        if self.row_expanded(prev_path):
+            return prev_iter
+        else:
+            return self.__find_previous_cat( prev_iter )    
     
     def __select_path(self, path):
         self.get_selection().select_path( path )
@@ -421,12 +531,33 @@ class CuemiacTreeView (gtk.TreeView):
     def __select_iter(self, iter):
         self.__select_path( self.get_model().get_path(iter) )
     
-    def __get_bottom_path(self):
+    def __find_bottom_path(self):
+        """
+        Find last item of last expanded category
+        """
         model = self.get_model()
         last_cat = len(model)-1
+        while (last_cat >= 0) and (not self.row_expanded( (last_cat,) )):
+            last_cat -= 1
+        if last_cat < 0:
+            # All categories are collapsed
+            return None
         last_cat_iter =  model.iter_nth_child(None, last_cat)
         last_cat_children = model.iter_n_children(last_cat_iter)-1
         return (last_cat, last_cat_children)
+    
+    def __find_top_path(self):
+        """
+        Find first item in first expanded category
+        """
+        model = self.get_model()
+        cat = 0
+        while (cat < len(model)) and (not self.row_expanded( (cat,) )):
+            cat += 1
+        if cat >= len(model):
+            # All categories are collapsed
+            return None
+        return (cat, 0)
 
 if gtk.pygtk_version < (2,8,0):    
     gobject.type_register (CuemiacTreeView)
